%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% WELOED.M: Welfare calculations.
%
% Matthew Gentzkow
% September 25, 2003
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	filekey 	string				key of files in current directory (OEDout### and run###) where ### is key 
%
%	
%
% FILES SAVED:
%	OEDwel### 
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function WELOED(filekey)

global NN NNtot GG CC KK DD PP PMAX ZZ CCW mf mz msee melast mis

% parameters
mstderr = 1;                        % number of simulation draws for standard error
postpos = 1;
postcompos = 2;
piweb = .1679;
piprint = .780919;
NPrint = 1780547;
%issuesperyear = [365-104 365 365-104];
%goodspi = [piprint piweb piprint];  % vector of profit per reported reader per day for each good
issuesperyear = [365-104 365];
goodspi = [piprint piweb];  % vector of profit per reported reader

% set filenames
inputfile1 = strcat('run',filekey);
inputfile2 = strcat('OEDest',filekey);
inputfile3 = strcat('OEDout',filekey);
outputfile = strcat('OEDwel',filekey);

% load input variables
cd 'M:\PROJECTS\PrintOnline\current run\input'
load(inputfile1,'Y1all','Y7all','Xall','CM','surveyweights','ascribed',...
    'excl','lhslist1','Mcons','Mincl','Bcons','Bex','R','r','buildparams')

cd 'M:\PROJECTS\PrintOnline\current run\results'
load(inputfile2,'bstar','headers','lhslist1','dL','L','dLc','Hc','SE1','S1','estparams','msteps');
load(inputfile3,'CDwTOT','CDwTOTse','CDwTOTcov','nchgood','D','Dse','Uall','mweights','msee','RD');
S = S1;
SE = SE1;

if (wrongbuildparams(buildparams))
    error('Parameters in data do not match those with which C-files were compiled');
end

[NNtot NN GG CC KK DD PMAX PP ZZ CCW] = extract(buildparams);
[UseDyn,ImpSamp,tolf0,tolff,tolx0,tolxf,mfe0,mfef,miter0,miterf,m0,mis,mf,mtest,msee,melast] = extract(estparams);
msee=100;

% temporarily set aside CDw
CDw = ones(NNtot,GG*GG);

% extract gam and tau parameters and calculate Uadd
gam = bstar(PP-1);
tau = bstar(PP-2);
Uadd = sum(CM*tau,2);
    
% random draws
if DD>0
%    RD = randn(NNtot*DD,msee);
else
    msee = 1;
    RD = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    MONTE-CARLO DRAWS ON BSTAR AND CDw    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if mstderr>1
    sprintf('monte-carlo draws')
    [T A] = transmats(R,r);
    RDsterr = randn(ZZ,mstderr);
    Sc = T'*S*T;
    Schol = chol(Sc)';
    bstarc = (bstar'*T)';
    bstarcmat = repmat(bstarc,1,mstderr) + (Schol * RDsterr);
    bstarmat = (bstarcmat'*T'+repmat(A',mstderr,1))';
    
    dNPmean = CDwTOT(postpos,postpos);
    dNWmean = CDwTOT(postpos,postcompos);
    dNcov = [CDwTOTcov(postpos,postpos) CDwTOTcov(postpos,postcompos);...
         CDwTOTcov(postcompos,postpos) CDwTOTcov(postcompos,postcompos)];
    RDsterr = randn(2,mstderr);
    dNchol = chol(dNcov)';
    dNmat = repmat([dNPmean;dNWmean],1,mstderr) + (dNchol * RDsterr);
else
    dNPmean = CDwTOT(postpos,postpos);
    dNWmean = CDwTOT(postpos,postcompos);
end

% define matrices to record things
alphamat = zeros(1,mstderr);
WelfareLossmat = zeros(GG,mstderr);
WelfareLossPermat = zeros(GG,mstderr);

for mm = 1:mstderr
    
    % use true parameter estimates for first column
    if mm>1
        b = bstarmat(:,mm);
        dNP = dNmat(1,mm);
        dNW = dNmat(2,mm);
    else
        b = bstar;
        dNP = dNPmean;
        dNW = dNWmean;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %    PRICE COEFFICIENTS          %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    sprintf('price coefficients -- normal')

    % see memo for calculation of alpha and why it doesn't include lamda
    alpha = -NPrint/(piprint*dNP + piweb*dNW);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    %    CONSUMER WELFARE     %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%

    sprintf('consumer surplus calculation')

    % calculate welfare loss from removing each good
    WelfareLoss = zeros(GG,1);
    U = utilities(b,excl,Xall',RD,CM',msee,NNtot);
    U = reshape(U,CC-1,msee*NNtot)';
    U = [zeros(msee*NNtot,1) U];
    Unews = U+repmat(Uadd',msee*NNtot,1);
    EUbefore = log(sum(exp(U),2));
    EUbeforenews = log(sum(exp(Unews),2));
    EUbefore = gam*EUbeforenews + (1-gam)*EUbefore;
    EUbefore = mean(reshape(EUbefore.*mweights,NNtot,msee),2);
    for ii = 1:GG
        Uafter = selectinmat(U',1-CM(:,ii))';
        Uafternews = selectinmat(Unews',1-CM(:,ii))';
	    EUafter = log(sum(exp(Uafter),2));
        EUafternews = log(sum(exp(Uafternews),2));
        EUafter = gam*EUafternews + (1-gam)*EUafter;
        EUafter = mean(reshape(EUafter.*mweights,NNtot,msee),2);
        WelfareLoss(ii) = sum(surveyweights.*(EUbefore-EUafter))./alpha;
    end
    WelfareLossPer = WelfareLoss./nchgood;

    alphamat(1,mm) = alpha;
    WelfareLossmat(:,mm) = WelfareLoss;
    WelfareLossPermat(:,mm) = WelfareLossPer;
end

dNP = dNPmean;
dNW = dNWmean;
alpha = -NPrint/(piprint*dNP + piweb*dNW);

alphase = sqrt(var(alphamat));
WelfareLoss = WelfareLossmat(:,1);
WelfareLossse = sqrt(var(WelfareLossmat(:,2:mstderr)'))';
WelfareLossPer = WelfareLossPermat(:,1);
WelfareLossPerse = sqrt(var(WelfareLossPermat(:,2:mstderr)'))';


%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    PRODUCER WELFARE     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

sprintf('producer surplus calculation')
revday = nchgood.*goodspi';
revyear = revday.*issuesperyear';
drevday = D.*repmat(goodspi,GG,1);
drevdayse = Dse.*repmat(goodspi,GG,1);
drevyear = drevday.*repmat(issuesperyear,GG,1);
drevyearse = drevdayse.*repmat(issuesperyear,GG,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    OPTIMAL PRICE        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

sprintf('Find optimal web price')
OPTIONS = optimset('Display','iter','MaxFunEvals',mfef,'TolFun',tolff,'TolX',tolxf,'GradObj','off');
price = fminunc('profitwebprice',.16,OPTIONS,0,Uall,CM,surveyweights,piweb,piprint,issuesperyear,postpos,postcompos,alpha,msee,mweights,gam,Uadd);
profitopt = -profitwebprice(price,0,Uall,CM,surveyweights,piweb,piprint,issuesperyear,postpos,postcompos,alpha,msee,mweights,gam,Uadd);
profitplot = zeros(21,1);
prices = [0:20]*.05-.5;
for ii = 1:21
    profitplot(ii) = -profitwebprice(prices(ii),0,Uall,CM,surveyweights,piweb,piprint,issuesperyear,postpos,postcompos,alpha,msee,mweights,gam,Uadd);
end
%price=0; profitopt=0; profitplot=0;

% check transaction costs
profitatzero = profitplot(11);
pricetc = zeros(20,1);
profitopttc = zeros(20,1);
mintc = 0; dif = 0;
for ii = 1:20
    tc = ii/100;
    pricetc(ii) = fminunc('profitwebprice',price,OPTIONS,tc,Uall,CM,surveyweights,piweb,piprint,issuesperyear,postpos,postcompos,alpha,msee,mweights,gam,Uadd);
    profitopttc(ii) = -profitwebprice(pricetc(ii),tc,Uall,CM,surveyweights,piweb,piprint,issuesperyear,postpos,postcompos,alpha,msee,mweights,gam,Uadd);
    dif = profitopttc(ii)-profitatzero;
    if mintc==0 & dif<0
        mintc = tc;
    end
end
%pricetc=0;
%profitopttc=0;


% save
save(outputfile,'alpha','alphase','WelfareLoss','WelfareLossse',...
                'WelfareLossPer','WelfareLossPerse',...
                'revday','revyear','drevday','drevdayse','drevyear','drevyearse','price','profitplot','profitopt',...
                'pricetc','profitopttc','mintc');