/**********************************************************************
 *
 * GETCPARAMETERS: Retrieve parameters from the oedcalc.h header file.
 *
 * Matthew Gentzkow
 * July 15, 2003
 * Online Editions
 * 
 * MATLAB prototype:

 * cparam = getcparameters()
 *
 * compiler command: 
 *	mex getcparameters.c oedcalc.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *
 * OUTPUTS:
 *	cparam		9				parameter vector
 *
 ***********************************************************************/

#include <math.h>
#include "mex.h"
#include "oedcalc.h"

#define NINPUTS 0
#define NOUTMAX 1

/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int nout[NOUTMAX], mout[NOUTMAX];
  char errstring[200];
  int ii;
  double *OPTRS[NOUTMAX];
  
  /*set nout and mout arrays with output dimensions */
  mout[0] = 9;			nout[0] = 1;
  
  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }

  OPTRS[0][0] = NN;
  OPTRS[0][1] = GG;
  OPTRS[0][2] = CC;
  OPTRS[0][3] = KK;
  OPTRS[0][4] = DD;
  OPTRS[0][5] = PMAX;
  OPTRS[0][6] = PP;
  OPTRS[0][7] = ZZ;
  OPTRS[0][8] = CCW;

}


