/*********************************************************************************************
 *
 * OEDCALC.C: Implementation of functions in oedcalc.h.
 *
 * Matthew Gentzkow
 * July 11, 2003
 * Online Editions
 *
 *********************************************************************************************/
 
#include <math.h>
#include "mex.h"
#include "oedcalc.h"


 /*********************************************************************************************
 *
 * CALCU
 *	
 *********************************************************************************************/

void calcu(double *U, double *b, double *excl, double *X, double *RD, double *CM, int minput, int ninput) 
{

static double *Uobs;
int baseX, baseEX, baseU, baseU2, baseRD, bcounter, bcounttemp;
int ii, jj, kk, mm;

Uobs = mxCalloc(ninput*(CC-1),sizeof(double));

/* initialize utilities to zero */
for(ii=0; ii<ninput*(CC-1); ii++) {
	Uobs[ii]=0;
}
for(ii=0; ii<minput*ninput*(CC-1); ii++) {
	U[ii]=0;
}

for(ii=0; ii<ninput; ii++) {
	baseU = ii*(CC-1);
	baseX = ii*KK;
	bcounter = 0;

	/* single goods */
	for(jj=0; jj<GG; jj++) {
		baseEX = (KK+DD)*jj;

		for(kk=0; kk<KK; kk++) {
			if(excl[baseEX+kk]==0) {
				Uobs[baseU+jj] += b[bcounter] * X[baseX+kk];
				bcounter++;
			}
		}

		/* add unobservables */
		if(DD>0){ for(mm=0; mm<minput; mm++) {
			bcounttemp = bcounter;
			baseU2 = mm*ninput*(CC-1) + baseU + jj;
			baseRD = mm*ninput*DD + ii*DD;
			U[baseU2] = Uobs[baseU + jj];
			for(kk=KK; kk<(KK+DD); kk++) {
				if(excl[baseEX+kk]==0) {
					U[baseU2] += b[bcounttemp] * RD[baseRD+kk-KK];
					bcounttemp++;
				}
			}
		}
		bcounter = bcounttemp;
		}
		else {
			U[baseU+jj] = Uobs[baseU+jj];
		}
		
	}

	/* combinations */
	for(jj=GG; jj<CC-1; jj++) {
		
		/* add in utilities of single goods */
		for(kk=0; kk<GG; kk++) {

			/* check if good kk is included in choice jj+1 */
			if(CM[GG*(jj+1)+kk]==1) {
				
				/* loop over random draws and add U of good kk to each */
				if(DD>0){ for(mm=0; mm<minput; mm++){
					U[mm*ninput*(CC-1)+baseU+jj] += U[mm*ninput*(CC-1)+baseU+kk];
				}
				}
				else {
					U[baseU+jj] += U[baseU+kk];
				}
			}
		}

		/* add combination-specific utilities (observables) */
		baseEX = (KK+DD)*jj;
		for(kk=0; kk<KK; kk++) {
			if (excl[baseEX+kk]==0) {
				Uobs[baseU+jj] += b[bcounter] * X[baseX+kk];
				bcounter++;
			}
		}

		/* add combination-specific utilities (unobservables) */
		if(DD>0){ for(mm=0; mm<minput; mm++) {
			bcounttemp = bcounter;
			baseU2 = mm*ninput*(CC-1) + baseU + jj;
			baseRD = mm*ninput*DD + ii*DD;
			U[baseU2] += Uobs[baseU+jj];
			for(kk=KK; kk<(KK+DD); kk++) {
				if (excl[baseEX+kk]==0) {
					U[baseU2] += b[bcounttemp] * RD[baseRD+kk-KK];
					bcounttemp++;
				}
			}
		}
		bcounter = bcounttemp;
		}
		else {
			U[baseU+jj] += Uobs[baseU+jj];
		}
		
	}
}

mxFree(Uobs);

}


 /*********************************************************************************************
 *
 * CALCUSINGLE:
 *
 *********************************************************************************************/
   
void calcusingle(double *U, double *b, double *excl, double *data, double *CM) 
{

double *X, *RD;

X = data;
RD = &data[KK];

calcu(U, b, excl, X, RD, CM, 1, 1);

}


 /*********************************************************************************************
 *
 * CALCNONLIN
 *	
 *********************************************************************************************/

void calcnonlin(double *coeff, double *dcoeff, double *gam, double *dgam, double *uadd, 
                double *b, double *CM)
{
double pi, pihat, dpi, tau, gamhat;
int ii, jj;

/* coeff if necessary */
if(coeff!=NULL) {
    pihat = b[PP-1];
    pi = exp(pihat)/(1+exp(pihat));
    calccoeff(coeff,pi);
}

/* gam and tau */
gamhat = b[PP-2];
*gam = exp(gamhat)/(1+exp(gamhat));
tau = b[PP-3];

/* uadd */
for(ii=1;ii<CC;ii++){
    uadd[ii-1]=0;
    for(jj=0;jj<GG;jj++){
        uadd[ii-1]+=tau*CM[GG*ii+jj];
    }
}

/* derivative terms if necessary */
if(dcoeff!=NULL & dgam!=NULL){
    dpi = pi/(1+exp(pihat));
    calcdcoeff(dcoeff,pi,dpi);
    *dgam = *gam*(1-*gam);
}

}


 /*********************************************************************************************
 *
 * CALCCOEFF
 *	
 *********************************************************************************************/

void calccoeff(double *coeff, double pi)

{

coeff[0] = pow((1-pi),4);
coeff[1] = pow((1-pi),3) * pi * 4;
coeff[2] = pow((1-pi),2) * pow(pi,2) * 6;
coeff[3] = (1-pi) * pow(pi,3) * 4;
coeff[4] = pow(pi,4);

}



 /*********************************************************************************************
 *
 * CALCDCOEFF
 *	
 *********************************************************************************************/

void calcdcoeff(double *dcoeff, double pi, double dpi)
{

int ii;

dcoeff[0] = -4 * pow(1-pi,3);
dcoeff[1] = 4 * pow(1-pi,3) - 12 * pi * pow(1-pi,2);
dcoeff[2] = 12 * pi * pow(1-pi,2) - 12 * (1-pi) * pow(pi,2);
dcoeff[3] = -4 * pow(pi,3) + 12 * (1-pi) * pow(pi,2);
dcoeff[4] = 4 * pow(pi,3);

/* account for dpi/dpihat */
for(ii=0; ii<5; ii++) { dcoeff[ii] *= dpi; } 

}


 /*********************************************************************************************
 *
 * FINDCONSTANTS
 *	
 *********************************************************************************************/

void findconstants(int *constants, double *excl, double *X)
{

int ii, jj, count;

count=-1;
for(ii=0;ii<GG;ii++) {
    for(jj=0;jj<KK;jj++){
        count+=1-(int)excl[ii*(KK+DD)+jj];
    }

    if(count>=0){constants[ii]=count;}
	else{constants[ii]=0;}
    
	for(jj=KK;jj<KK+DD;jj++){
        count+=1-(int)excl[ii*(KK+DD)+jj];
    }
}

/* test to make sure constants are in the right place*/
for(ii=0;ii<NN;ii++){
    if(X[ii*KK+KK-1]!=1){
        mexErrMsgTxt("Error finding constant terms.");
    }
}

}


 /*********************************************************************************************
 *
 * EXPAND
 *	
 *********************************************************************************************/

void expand(double *b, double *bc, double *T, double *A)

{

int ii, jj;

for(ii=0; ii<PP; ii++) {
    b[ii]=0;  
    for(jj=0; jj<ZZ; jj++) {
        b[ii] = b[ii] + bc[jj]*T[PP*jj+ii];
    }
	b[ii]+=A[ii];
}

}



 /*********************************************************************************************
 *
 * CALCNONLINSM
 *	
 *********************************************************************************************/

void calcnonlinsm(double *gam, double *uadd, double *b, double *CM)

{
double *coeff=NULL;
double *dcoeff=NULL;
double *dgam=NULL;

calcnonlin(coeff,dcoeff,gam,dgam,uadd,b,CM);


}


