/**********************************************************************
 *
 * OEDCALCBIG.C: Implementation of functions in oedcalcbig.h.
 *
 * Matthew Gentzkow
 * July 11, 2003
 * Online Editions
 *
 ***********************************************************************/
 
#include <math.h>
#include "mex.h"
#include "oedcalc.h"
#include "oedcalcbig.h"

/***********************************************************************************************
 *
 * P1ALL7: 
 *
 *********************************************************************************************/

void p1all7(double *P7all, int c1, double *P1, double *Bcons, double *Bex, double *coeff)

{

double Prun[CC], Prunnew[CC], Pterm[5*CC];
int ii,jj,day,newii;

for (ii=0;ii<CC;ii++)	{Prun[ii]=0;}
for (ii=0;ii<5*CC;ii++)	{Pterm[ii]=0;}

/* day 0 */
Prun[c1]=P1[c1];
Pterm[c1]=Prun[c1];

/* days 1-4 */
/* loops are for starting at ii, choosing jj, and ending up at newii */
for(day=1;day<5;day++){
    
	for(ii=0;ii<CC;ii++){Prunnew[ii]=0;}

	for(ii=0;ii<CC;ii++){ if(Prun[ii]!=0){
		for(jj=0;jj<CC;jj++){
			newii = Bex[CC*ii+jj];
			Prunnew[newii] += Prun[ii]*P1[jj];
        }
    }
	}

    for(ii=0;ii<CC;ii++){Prun[ii]=Prunnew[ii];}
    for(ii=0; ii<CC; ii++) {
		Pterm[CC*day+ii] = Prun[ii];
	}
}

for(ii=0; ii<CC; ii++) {P7all[ii]=0;}

for(day=0;day<5;day++){
	for (ii=0; ii<CC; ii++) {
		P7all[ii] += coeff[day] * Pterm[CC*day+ii];
	}
}
}
 


/***********************************************************************************************
 *
 * PDP1ALL7: 
 *
 *********************************************************************************************/

void pdp1all7(double *P7all, double *dP7all, int c1, double *P1, double *dP1, double *Bcons, 
                double *Bex, double *coeff, double *dcoeff)

{

double Prun[CC], Prunnew[CC], dPrun[CC*PP], dPrunnew[CC*PP], Pterm[5*CC], dPterm[5*CC*PP];
int ii,jj,rr,day,newii;

for (ii=0;ii<CC;ii++)	{Prun[ii]=0;}
for (ii=0;ii<CC*PP;ii++)	{dPrun[ii]=0;}
for (ii=0;ii<5*CC;ii++)	{Pterm[ii]=0;}
for (ii=0;ii<5*PP*CC;ii++){dPterm[ii]=0;}

/* day 0 */
Prun[c1]=P1[c1];
for (ii=0; ii<PP-1; ii++){
	dPrun[c1*PP+ii] = dP1[c1*PP+ii];
}
Pterm[c1]=Prun[c1];
for (ii=0; ii<PP-1; ii++){
	dPterm[c1*PP+ii] = dPrun[c1*PP+ii];
}

/* days 1-4 */
/* loops are for starting at ii, choosing jj, and ending up at newii */
for(day=1;day<5;day++){
    
	for(ii=0;ii<CC;ii++){Prunnew[ii]=0;}
	for(ii=0;ii<CC*PP;ii++){dPrunnew[ii]=0;}

	for(ii=0;ii<CC;ii++){ if(Prun[ii]!=0){
		for(jj=0;jj<CC;jj++){
			newii = Bex[CC*ii+jj];
			Prunnew[newii] += Prun[ii]*P1[jj];
			for(rr=0; rr<PP-1; rr++) {
				dPrunnew[newii*PP+rr] += dPrun[ii*PP+rr]*P1[jj] + Prun[ii]*dP1[jj*PP+rr];
			} 
        }
    }
	}

    for(ii=0;ii<CC;ii++){Prun[ii]=Prunnew[ii];}
	for(ii=0;ii<CC*PP;ii++){dPrun[ii]=dPrunnew[ii];}
    for(ii=0; ii<CC; ii++) {
		Pterm[CC*day+ii] = Prun[ii];
		for(jj=0;jj<(PP-1);jj++){dPterm[CC*PP*day+PP*ii+jj] = dPrun[ii*PP+jj];}
	}
}

for(ii=0; ii<CC; ii++) {P7all[ii]=0;}
for(ii=0; ii<PP*CC; ii++) {dP7all[ii]=0;}

for(day=0;day<5;day++){

	for (ii=0; ii<CC; ii++) {
		P7all[ii] += coeff[day] * Pterm[CC*day+ii];
		for (jj=0; jj<PP-1; jj++) {
			dP7all[PP*ii+jj] += coeff[day] * dPterm[CC*PP*day+PP*ii+jj];
		}
		dP7all[PP*ii+(PP-1)] += dcoeff[day] * Pterm[CC*day+ii];
	}
}
}


/***********************************************************************************************
 *
 * CALCP7BIG: 
 *
 *********************************************************************************************/



/*void calcp7big(double *P7big, double *b, double *excl, double *Y1, double *Y7, double *X,
		double *RD, double *weights, double *CM,double *Cincl, double *Bcons,
		double *Mcons, double *Mincl, double *Bex, int m)

{

double P1[CC], coeff[5], P7all[CC] ;
double *U;
double pihat, pi, factor;
int c7, countcc, uindex;
int ii, jj, kk;

/* allocate U */
/*U = mxCalloc(m*NN*(CC-1),sizeof(double));

/* calculate U */
/*calcu(U,b,excl,X,RD,CM,m);

/* break out parameters */
/*pihat = b[PP-1];
pi = exp(pihat)/(1+exp(pihat));
calccoeff(coeff,pi);

/* initialize P7big */
/*for (ii=0; ii<NN*CCW; ii++) {P7big[ii]=0;}

/* main evaluation routine */
/*for(ii=0; ii<NN; ii++) {

	for(jj=0; jj<m; jj++) {	
		countcc = 0;
		uindex = jj*NN*(CC-1) + ii*(CC-1);
		calcp1(P1,&U[uindex]);
		for(kk=0; kk<CC; kk++) {
			/* check that this value of kk is actually included in some choice */
			/*if(Cincl[countcc]==kk) {
				p1all7(P7all,kk,P1,Bcons,Mcons,Mincl,Bex,coeff);
			}
			/* go through and write only elements of dP7all that are in Cincl table */
			/*while(Cincl[countcc]==kk) {
				c7 = Cincl[CCW+countcc];
				P7big[ii*CCW+countcc] += P7all[c7] * weights[jj*NN+ii];
				countcc++;
			}
		}

	}
}

factor = 1/(double)m;
for(ii=0; ii<NN*CCW; ii++) {P7big[ii] *= factor;}
	
}


/***********************************************************************************************
 *
 * CALCPDP7BIG: 
 *
 *********************************************************************************************/

/*void calcpdp7big(double *P7big, double *dP7big, double *b, double *excl, double *Y1, 
						double *Y7, double *X, double *RD, double *weights, double *CM, double *Cincl,
						double *Bcons, double *Mcons, double *Mincl, double *Bex, int m)

{

double P1[CC], dP1[PP*CC], coeff[5], dcoeff[5], P7all[CC], dP7all[PP*CC];
double pihat, pi, dpi, factor;
double *U;
int c7, countcc, uindex, xindex, rdindex;
int ii, jj, kk, mm;

/* allocate U */
/*U = mxCalloc(m*NN*(CC-1),sizeof(double));

/* calculate U */
/*calcu(U,b,excl,X,RD,CM,m);

/* break out parameters */
/*pihat = b[PP-1];
pi = exp(pihat)/(1+exp(pihat));
dpi = pi/(1+exp(pihat));
calccoeff(coeff,pi);
calcdcoeff(dcoeff,pi,dpi);

/* initialize dP7big and P7big */
/*for (ii=0; ii<NN*CCW*PP; ii++) {dP7big[ii]=0;}
for (ii=0; ii<NN*CCW; ii++) {P7big[ii]=0;}

/* main evaluation routine */
/*for(ii=0; ii<NN; ii++) {

	for(jj=0; jj<m; jj++) {	
		countcc = 0;
		uindex = jj*NN*(CC-1) + ii*(CC-1);
		xindex = ii*KK;
		rdindex = jj*NN*DD + ii*DD;
		calcp1(P1,&U[uindex]);

		calcdp1(dP1,P1,b,excl,&X[xindex],&RD[rdindex],CM);
		for(kk=0; kk<CC; kk++) {
			/* check that this value of kk is actually included in some choice */
			/*if(Cincl[countcc]==kk) {
				pdp1all7(P7all,dP7all,kk,P1,dP1,Bcons,Mcons,Mincl,Bex,coeff,dcoeff);
			}
			/* go through and write only elements of P7all and dP7all that are in Cincl table */
			/*while(Cincl[countcc]==kk) {
				c7 = Cincl[CCW+countcc];
				P7big[ii*CCW+countcc] += P7all[c7] * weights[jj*NN+ii];
				for(mm=0; mm<PP; mm++) {dP7big[mm*NN*CCW+ii*CCW+countcc] += dP7all[c7*PP + mm]*weights[jj*NN+ii];}
				countcc++;
			}
		}

	}
}

factor = 1/(double)m;
for(ii=0; ii<NN*CCW*PP; ii++) { dP7big[ii] *= factor; }
for(ii=0; ii<NN*CCW; ii++) { P7big[ii] *= factor; }
	
}

*/

