/***************************************************************************
 *
 * OEDOBJECTIVES.C: Implementation of functions in oedobjectives.h.
 *
 * Matthew Gentzkow
 * November 27, 2004
 * Online Editions
 *
 ****************************************************************************/

#include <math.h>
#include "mex.h"
#include "oedcalc.h"
#include "oedcalcprob.h"
/*#include "oedcalcbig.h"*/
#include "oedobjectives.h"


/***********************************************************************************************
 *
 * LIKELIHOOD: 
 *
 *********************************************************************************************/
void likelihood( double *L, double *dLc, double *bc, int minput, double *excl, double *Y1, 
				double *Y7, double *X, double *RD, double *CM, double *Bcons, double *Bex, 
                double *T, double *A, double *weights) 
{

double b[PP], dL[PP], dLi[PP], uadd[CC-1], coeff[5], dcoeff[5];
double *P, *dP, *P1, *dP1, *U, *obsweights;
double Li, gam, dgam;
int constants[GG],c1;
int ii, jj, kk;

/* decide whether we're using 7-day or 1-day data */
int sevenday = Y7==NULL ? 0:1;

/* decide whether we're supposed to calculate derivatives */
int calcderiv = dLc==NULL ? 0:1;

/* expand parameter vector */
expand(b,bc,T,A);

/* check for no unobservables case where m==0 */
if(minput==0) {minput=1;}

/* U, P7 must be allocated explicitly because m is variable */
U = mxCalloc(minput*NN*(CC-1),sizeof(double));
P = mxCalloc(minput,sizeof(double));
obsweights = mxCalloc(minput,sizeof(double));

/* initialize pointers for 1-day and deriv cases */
P1 = NULL;
dP1=NULL;
dP=NULL;
if(sevenday==0){
    P1 = mxCalloc(minput*CC,sizeof(double));
}
if(calcderiv==1){
    dP = mxCalloc(minput*PP,sizeof(double));
    if(sevenday==0){
        dP1 = mxCalloc(minput*CC*PP,sizeof(double));
    }
}

/* initialize L and dL */
*L=0;
for(ii=0; ii<PP; ii++) { dL[ii]=0; }

/* calculate U and parameters */
calcu(U,b,excl,X,RD,CM,minput,NN);
calcnonlin(coeff,dcoeff,&gam,&dgam,uadd,b,CM);
findconstants(constants,excl,X);

/* main evaluation routine */
for(ii=0; ii<NN; ii++) {
    for(jj=0;jj<minput;jj++){
        P[jj]=0;
        obsweights[jj] = weights[NN*jj+ii];
    }
    
    if(sevenday==1) {
        calcp7all(P,dP,U,b,excl,X,RD,CM,Bcons,Bex,coeff,dcoeff,Y1,Y7,uadd,constants,gam,dgam,minput,ii);
    } else {
        calcp1all(P1,dP1,U,b,excl,X,RD,CM,uadd,constants,gam,dgam,minput,ii);
        /* copy observed choice from P1 and dP1 into P and dP */
        c1=(int)Y1[ii];
        for(jj=0;jj<minput;jj++){
            P[jj]=P1[jj*CC+c1];
            if(calcderiv==1){
                for(kk=0;kk<PP;kk++){
                    dP[jj*PP+kk]=dP1[jj*CC*PP+c1*PP+kk];
                }
            }       
        }
    }
    
    /* calculate Li and dLi */
    if(calcderiv==1){
    	calcL(&Li,dLi,P,dP,obsweights,minput);
    } else {
    	calcL(&Li,NULL,P,NULL,obsweights,minput);
    }
    
    /* add to total */
	*L -= Li;    
    if(calcderiv==1){
    	for (jj=0; jj<PP; jj++) {
        	dL[jj] -= dLi[jj];
        }
    }
}

/* map dL back to dLc */
if(calcderiv==1){
    for (ii=0; ii<ZZ; ii++) {
        dLc[ii]=0;
        for (jj=0; jj<PP; jj++) {
            dLc[ii] += T[PP*ii+jj]*dL[jj];
        }
    }
}

}





/*********************************************************************************************
 *
 * MSMOBJECTIVE: 
 *
 **********************************************************************************************/

/*void msmobjective(double *Gn, double *dGnc, double *bc, double *excl, double *Z, double *Y1, 
		double *Y7, double *Cdum, double *X, double *RD, double *weights, double *T, 
		double *A, double *CM, double *Cincl, double *Bcons, double *Mcons, 
		double *Mincl, double *Bex, int m)
{

static double P7big[NN*CCW], dP7big[PP*NN*CCW];
double M[PP], dM[PP*PP], dGn[PP], b[PP];
int ii, jj, kk;*/

/* expand parameter vector */
/*expand(b,bc,T,A);*/

/* call function to generate vectors of probabilities and derivatives */

/*calcpdp7big(P7big,dP7big,b,excl,Y1,Y7,X,RD,weights,CM,Cincl,Bcons,Mcons,Mincl,Bex,m);

*Gn = 0;

for (ii=0; ii<PP; ii++) { */

	/* calculate M and Gn */
/*	M[ii]=0; 
	for (jj=0; jj<NN*CCW; jj++) {
		M[ii]+=(Cdum[jj]-P7big[jj])*Z[NN*CCW*ii+jj];
	}
	/*M[ii] /= (double)NN;*/
/*	*Gn += pow(M[ii],2);
	
	/* calculate dM */
/*	for (jj=0; jj<PP; jj++) {
		dM[ii*PP+jj]=0;
		for (kk=0; kk<NN*CCW; kk++) {
			dM[ii*PP+jj] += -dP7big[NN*CCW*jj+kk]*Z[NN*CCW*ii+kk];
		}
		/*dM[ii*PP+jj] /= (double)NN;*/
/*	}
}

/* calculate dGn */
/*for (ii=0; ii<PP; ii++) {
	dGn[ii] = 0;
	for (jj=0; jj<PP; jj++) {
		dGn[ii] += 2*M[jj]*dM[jj*PP+ii];
	}
}

/* map dGn back to dGnc */
/*for (ii=0; ii<ZZ; ii++) {
	dGnc[ii]=0;
	for (jj=0; jj<PP; jj++) {
		dGnc[ii] += T[PP*ii+jj]*dGn[jj];
	}
}	

}


/*********************************************************************************************
 *
 * MSMOBJECTIVEND: 
 *
 ************************************************************************************************/

/*void msmobjectivend(double *Gn, double *bc, double *excl, double *Z, double *Y1, double *Y7, 
		double *Cdum, double *X, double *RD, double *weights, double *T, double *A, 
		double *CM, double *Cincl, double *Bcons, double *Mcons, double *Mincl,double *Bex, int m)

{

double P7big[NN*CCW], M[PP], b[PP];
int ii, jj;

/* expand parameter vector */
/*expand(b,bc,T,A);

calcp7big(P7big,b,excl,Y1,Y7,X,RD,weights,CM,Cincl,Bcons,Mcons,Mincl,Bex,m);

*Gn = 0;
for (ii=0; ii<PP; ii++) { 

	/* calculate M and Gn */
/*	M[ii]=0; 
	for (jj=0; jj<NN*CCW; jj++) {
		M[ii]+=(Cdum[jj]-P7big[jj])*Z[NN*CCW*ii+jj];
	}
	/*M[ii] /= (double)NN;*/
/*	*Gn += pow(M[ii],2);
}

}
*/

