%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% BUILDY.M: Function to generate Y1 and Y7 from matrices of dummy variables for the
%           choice of each good.
%
% Matthew Gentzkow
% July 17, 2003
% Online Editions
%
% note: a*b*c in dimension means vectors stacked as a groups of b groups of c
%
% INPUTS:
%	GG			scalar				number of goods
%	NN			scalar				number of consumers
%	Ymat1		NN x GG				ixj=1 if consumer i chose good j yesterday
%	Ymat7		NN x GG				ixj=1 if consumer i chose good j in last 7 days
%	
% OUTPUTS:
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	CM			CC*GG				i*j=1 if good j included in choice i
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Y1,Y7,CM] = buildY(GG,NN,Ymat1,Ymat7)

Y1 = zeros(NN,1);
Y7 = zeros(NN,1);
maxc = 2^GG;
CM = [zeros(1,GG); eye(GG); 9*ones(maxc-GG-1,GG)];
totc = GG+1;
match = 0;
for i=1:NN
    index = findinmat(Ymat1(i,:),CM(1:totc,:));
    if index==-1
        totc=totc+1;
        CM(totc,:)=Ymat1(i,:);
        Y1(i)=totc;
    else
        Y1(i)=index;
    end
end
for i = 1:NN
    index = findinmat(Ymat7(i,:),CM(1:totc,:));
    if index==-1
        totc=totc+1;
        CM(totc,:)=Ymat7(i,:);
        Y7(i)=totc;    
    else
        Y7(i)=index;
    end
end    
Y1 = Y1-1;
Y7 = Y7-1;
CM = CM(1:totc,:);
    