%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% GENIS.M: Generate random draws by importance sampling.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	X			NN x KK				observable rhs vars
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	CM			CC x GG				i,j=1 if good j included in choice i
%	Bcons		CC x CC				i,j=1 if choice j potentially consistent with choice i
%	Bex			CC x CC				i,j=k where 1-day choice i takes overall choice from j to k
%	b0			PP					initial estimate of parameter vector
%	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
%	minput		scalar				number of random draws to generate
%
%	
% OUTPUTS:
%	RDis  		NN*DD x mf  		observable rhs variables and random draws
%	weights		NN x mf				importance sampling weights for each observation
%   P0          NN*(CC-1) x mf      probabilities of each choice at each draw
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [RDis, weights, P0] = genis(X,Y1,Y7,CM,Bcons,Bex,b0,excl,minput,Pavg0)

global mis KK DD NN

% calculate preliminary probabilities Pavg0
RD = randn(NN*DD,mis);
sprintf('Calculate Pavg0')
tic
Pavg0 = probabilities7(X',Y1,Y7,RD,b0,excl,CM',Bcons,Bex,mis,NN);
Pavg0 = mean(Pavg0,2);
toc

% seeds for C random number generator
IJseed = sum(clock);
KLseed = 2*sum(clock);
if(IJseed>31328 | KLseed>30081), error('seed in genis out of range'); end

% generate draws RD and associated probabilities of observed choice P0
sprintf('Generate importance sampling draws')
tic
[P0,RDis] = isdraws(b0,excl,minput,Y1,Y7,X',CM',Bcons,Bex,IJseed,KLseed);
toc

% calculate weights
weights = repmat(Pavg0,1,minput)./P0;

