%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% REVERSESELECT.M: Reverses the effect of selectinmat (sort of). Takes a matrix M and
%				expands it according to a vector V. For each row i of V, the
%				corresponding row of the resulting matrix is all zeros if V==0
%				or equal to a row of M if V==1.
%
% Matthew Gentzkow
% October 8, 2002
% Online Editions
%
% note: a*b*c in dimension means vectors stacked as a groups of b groups of c
%
% INPUTS:
%	M			matrix
%	V			V(i)=1 if row i to be retained, =0 otherwise
%	
% OUTPUTS:
%	Mnew		matrix of only the selected rows of M
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Mnew = reverseselect(M,V)

[n,k] = size(M);
bign = length(V);
if n ~= sum(V), error('Matrix and vector of different sizes'); end
M = [M;zeros(1,k)];

indices = [];
counter = 1;
for i = 1:bign
	if V(i)==1
		indices = [indices;counter];
		counter = counter+1;
	else
		indices = [indices;n+1];
	end
end

Mnew = M(indices,:);
