/*********************************************************************************************
 *
 * OEDCALCPROB.C: Implementation of functions in oedcalcprob.h.
 *
 * Matthew Gentzkow
 * November 27, 2004
 * Online Editions
 *
 *********************************************************************************************/
 
#include <math.h>
#include "mex.h"
#include "oedcalc.h"
#include "oedcalcprob.h"


 /*********************************************************************************************
 *
 * CALCP1ALL:
 *
 *********************************************************************************************/

void calcp1all(double *P1, double *dP1, double *U, double *b, double *excl, double *X, double *RD, double *CM, 
				  double *uadd, int *constants, double gam, double dgam, int m, int consindex)
{

int uindex, xindex, rdindex;
int jj;

/* decide whether we're supposed to calculate the derivative */
int calcderiv = dP1==NULL ? 0:1;

uindex = consindex*(CC-1);
xindex = consindex*KK;
rdindex = consindex*DD;

for(jj=0; jj<m; jj++) {	
    if(calcderiv==1){
        calc1tg(&P1[jj*CC],&dP1[jj*CC*PP],&U[uindex+jj*NN*(CC-1)],b,excl,
            &X[xindex],&RD[rdindex+jj*NN*DD],CM,uadd,constants,gam,dgam);
    }else{
        calc1tg(&P1[jj*CC],NULL,&U[uindex+jj*NN*(CC-1)],b,excl,
            &X[xindex],&RD[rdindex+jj*NN*DD],CM,uadd,constants,gam,dgam);
    }
}
}


/*********************************************************************************************
 *
 * CALCP7ALL:
 *
 *********************************************************************************************/

void calcp7all(double *P7, double *dP7, double *U, double *b, double *excl, double *X, double *RD, double *CM, 
				   double *Bcons, double *Bex, double *coeff, double *dcoeff, double *Y1, double *Y7, 
                   double *uadd, int *constants, double gam, double dgam, int m, int consindex)
{

double P1[CC], dP1[PP*CC];
int c1, c7, uindex, xindex, rdindex;
int jj;

/* decide whether we're supposed to calculate the derivative */
int calcderiv = dP7==NULL ? 0:1;

c1 = (int)Y1[consindex];
c7= (int)Y7[consindex];
uindex = consindex*(CC-1);
xindex = consindex*KK;
rdindex = consindex*DD;

for(jj=0; jj<m; jj++) {	
    if(calcderiv==1){
        calc1tg(P1,dP1,&U[uindex+jj*NN*(CC-1)],b,excl,&X[xindex],&RD[rdindex+jj*NN*DD],
            CM,uadd,constants,gam,dgam);
        calcpdp7(&P7[jj],&dP7[jj*PP],c1,c7,P1,dP1,Bcons,Bex,coeff,dcoeff);
    } else {
        calc1tg(P1,NULL,&U[uindex+jj*NN*(CC-1)],b,excl,&X[xindex],&RD[rdindex+jj*NN*DD],
            CM,uadd,constants,gam,dgam);
        calcpdp7(&P7[jj],NULL,c1,c7,P1,NULL,Bcons,Bex,coeff,dcoeff);
    }
}

}


/*********************************************************************************************
 *
 * CALC1TG: 
 *
 *********************************************************************************************/
   
void calc1tg(double *P1, double *dP1, double *U, double *b, double *excl, double *X, double *RD, 
                double *CM, double *uadd, int *constants, double gam, double dgam) 
{

double Unews[CC-1], P1news[CC], dP1news[CC*PP];
int ii, jj;

/* decide whether we're supposed to calculate the derivative */
int calcderiv = dP1==NULL ? 0:1;

 /* calculate Unews*/
for(ii=0;ii<CC-1;ii++){
    Unews[ii]=U[ii]+uadd[ii];
}

calcp1(P1,U);
calcp1(P1news,Unews);
if(calcderiv==1){
    calcdp1(dP1,P1,b,excl,X,RD,CM);
    calcdp1(dP1news,P1news,b,excl,X,RD,CM);
}

for(ii=0;ii<CC;ii++){
    if(calcderiv==1){
        /* derivatives w.r.t. linear parameters */
        for(jj=0; jj<PP-3; jj++) {dP1[ii*PP+jj] = (1-gam)*dP1[ii*PP+jj]+gam*dP1news[ii*PP+jj];}

        /* derivative w.r.t. tau */
        dP1[ii*PP+PP-3] = 0;
        for(jj=0; jj<GG; jj++){
            dP1[ii*PP+PP-3]+=gam*dP1news[ii*PP+constants[jj]];
        }
        
        /* derivative w.r.t. gam */
        dP1[ii*PP+PP-2] = dgam*(P1news[ii]-P1[ii]); 
    }
    P1[ii] = (1-gam)*P1[ii]+gam*P1news[ii];
}
}


/*********************************************************************************************
 *
 * CALCP1: 
 *
 *********************************************************************************************/
   
void calcp1(double *P1, double *U) 
{

register double Usum;
double eU[CC-1];
int ii;

Usum = 1;		/* starting val is utility for choice 0 */
        
/* calculate eU */
for(ii=0; ii<CC-1; ii++) {
	eU[ii] = exp(U[ii]);
	Usum += eU[ii];
}
                
/* calculate P1 */
P1[0]=1;
for(ii=1; ii<CC; ii++) {
	P1[ii]=eU[ii-1]/Usum;
	P1[0] -= P1[ii];
}

/* make sure there is no rounding error that makes P1[0] negative */
if(P1[0]<0){ P1[0]=0; }

}



/***********************************************************************************************
 *
 * CALCDP1: 
 *
 *********************************************************************************************/

void calcdp1(double *dP1, double *P1, double *b, double *excl, double *X, double *RD, 
                double *CM)
{

double sumP1[GG];
int base1, base2, bcounter, numnonlin;
int ii, jj, kk;

bcounter = 0;

/* calculate number of nonlinear parameters */
numnonlin=PMAX-(CC-1)*(KK+DD);

/* calculate sum of probabilities for each good */
for (ii=0; ii<GG; ii++) {
    sumP1[ii]=0;
    for (jj=0; jj<CC; jj++) {
        if (CM[GG*jj+ii]==1) {
            sumP1[ii] += P1[jj];
        }
    }
}


for (ii=0; ii<CC; ii++) {
	base1 = PP*ii;

	/* calculate derivatives: single good parameters */
	for (jj=0; jj<GG; jj++) {
		base2 = (KK+DD)*jj;
        if (CM[GG*ii+jj]==1) {
            for (kk=0; kk<KK; kk++) {
				if (excl[base2+kk]==0) {
		            dP1[bcounter] = X[kk]*P1[ii]*(1-sumP1[jj]);
					bcounter++;
				}
            }
			if(DD>0){ for (kk=KK; kk<(KK+DD); kk++) {
				if (excl[base2+kk]==0) {
		            dP1[bcounter] = RD[kk-KK]*P1[ii]*(1-sumP1[jj]);
					bcounter++;
				}
            }
			}

        }
        else {
            for (kk=0; kk<KK; kk++) {
				if (excl[base2+kk]==0) {
		             dP1[bcounter] = -X[kk]*P1[ii]*sumP1[jj];
					 bcounter++;
				}
            }
            if(DD>0){ for (kk=KK; kk<(KK+DD); kk++) {
				if (excl[base2+kk]==0) {
		             dP1[bcounter] = -RD[kk-KK]*P1[ii]*sumP1[jj];
					 bcounter++;
				}
            }
			}

        }
    }

	/* calculate derivatives: combinations */
	for (jj=GG+1; jj<CC;jj++) {
		base2 = (KK+DD)*(jj-1);
        if (jj==ii) {
			for (kk=0; kk<KK; kk++) {
				if (excl[base2+kk]==0) {
					dP1[bcounter] = X[kk]*P1[ii]*(1-P1[ii]);
					 bcounter++;
				}
			}
			if(DD>0){ for (kk=KK; kk<(KK+DD); kk++) {
				if (excl[base2+kk]==0) {
					dP1[bcounter] = RD[kk-KK]*P1[ii]*(1-P1[ii]);
					 bcounter++;
				}
			}
			}

		}
		else {
			for (kk=0; kk<KK; kk++) {
				if (excl[base2+kk]==0) {
					dP1[bcounter] = -X[kk]*P1[ii]*P1[jj];
					 bcounter++;
				}
			}
			if(DD>0){ for (kk=KK; kk<(KK+DD); kk++) {
				if (excl[base2+kk]==0) {
					dP1[bcounter] = -RD[kk-KK]*P1[ii]*P1[jj];
					 bcounter++;
				}
			}
			}
		}
    }

	bcounter+=numnonlin; /* increment for nonlinear parameters */

}

}


/***********************************************************************************************
 *
 * CALCPDP7:
 *
 *********************************************************************************************/


void calcpdp7(double *P7, double *dP7, int c1, int c7, double *P1, double *dP1, double *Bcons, 
					   double *Bex, double *coeff, double *dcoeff)

{

/* decide whether we're supposed to calculate the derivative */
int calcderiv = dP7==NULL ? 0:1;

double Prun[CC], Prunnew[CC], Pterm[5];
double dPrun[CC*PP], dPrunnew[CC*PP], dPterm[5*PP];
int ii,jj,tt,rr,day;

/* initialize Prun[] and Pterm[] */
for (ii=0;ii<CC;ii++)	{Prun[ii]=0;}
for (ii=0;ii<5;ii++)	{Pterm[ii]=0;}
Prun[c1]=P1[c1];
if(c1==c7) {
    Pterm[0]=Prun[c1];
}

/* initialize dPrun and dPterm if necessary */
if(calcderiv==1) {
	for (ii=0;ii<CC*PP;ii++)	{dPrun[ii]=0;}
	for (ii=0;ii<5*PP;ii++)	{dPterm[ii]=0;}
    for (ii=0; ii<PP-1; ii++){
        dPrun[c1*PP+ii] = dP1[c1*PP+ii];
    }
    if(c1==c7) {
        for (ii=0; ii<PP-1; ii++){
            dPterm[ii] = dPrun[c1*PP+ii];
        }
    }
}


/* days 1-4 */
/* loops are for probability of going from ii to jj via choice tt (for each day) */
for(day=1;day<5;day++){
    
	for(ii=0;ii<CC;ii++){Prunnew[ii]=0;}
    if(calcderiv==1){
        for(ii=0;ii<CC*PP;ii++){dPrunnew[ii]=0;}
    }

	for(ii=0;ii<CC;ii++){ if(Prun[ii]!=0){
		for(jj=0;jj<CC;jj++){ if(Bcons[ii*CC+jj]==1 & Bcons[jj*CC+c7]==1){	
			for(tt=0;tt<CC;tt++){ if(Bex[ii*CC+tt]==jj){
						
				Prunnew[jj] += Prun[ii]*P1[tt];
                if(calcderiv==1){
                    for(rr=0; rr<PP-1; rr++) {
                        dPrunnew[jj*PP+rr] += dPrun[ii*PP+rr]*P1[tt] + Prun[ii]*dP1[tt*PP+rr];
                    }
                }

            }
            }
        }
		}
    }
    }

    for(ii=0;ii<CC;ii++){Prun[ii]=Prunnew[ii];}
    Pterm[day] = Prun[c7];
    if(calcderiv==1){
        for(ii=0;ii<CC*PP;ii++){dPrun[ii]=dPrunnew[ii];}
        for(ii=0;ii<(PP-1);ii++){dPterm[day*PP+ii] = dPrun[c7*PP+ii];}
    }
}

*P7=0;
for(day=0;day<5;day++){
	*P7 += coeff[day] * Pterm[day];
}

if(calcderiv==1){
    for(ii=0; ii<PP; ii++) {dP7[ii]=0;}
    for(day=0;day<5;day++){
        for (ii=0; ii<PP-1; ii++) {
            dP7[ii] += coeff[day] * dPterm[day*PP + ii];
        }

        dP7[PP-1] += dcoeff[day] * Pterm[day];
    }
}

}


 /*********************************************************************************************
 *
 * CALCL
 *	
 *********************************************************************************************/

void calcL(double *Li, double *dLi, double *P7, double *dP7, double *weights, int m)

{

double P7sum, dP7sum[PP], Pavg, dPavg[PP], ctemp, cterm,  dctemp[PP], dcterm[PP];
int ii, jj;

/* decide whether we're supposed to calculate derivatives */
int calcderiv = (dLi==NULL | dP7==NULL) ? 0:1;

P7sum=0;
cterm=0; ctemp=0; P7sum=0;
if(calcderiv==1){
    for(ii=0;ii<PP;ii++){dP7sum[ii]=0; dPavg[ii]=0; dctemp[ii]=0; dcterm[ii]=0;}
}

if(DD>0) {
    /* scale by weights */
	for(ii=0; ii<m; ii++) {
		P7[ii] *= weights[ii];
	}

	/* sum P7 */
	for(ii=0;ii<m;ii++) {
		P7sum += P7[ii];
	}
	
	/* averages */
	Pavg = (1/(double)m)*P7sum;

    for (ii=0; ii<m; ii++){
		ctemp += pow(P7[ii]-Pavg,2);
	}

    if(calcderiv==1) {        
        /* scale by weights */
        for(ii=0; ii<m; ii++) {
            for(jj=0; jj<PP; jj++) {dP7[PP*ii+jj] *= weights[ii];}
        }

        /* sum P7 */
        for(ii=0;ii<m;ii++) {
            for (jj=0; jj<PP; jj++) {dP7sum[jj] += dP7[PP*ii+jj];}
        }

        /* averages */
        for (ii=0; ii<PP; ii++) {dPavg[ii] = (1/(double)m)*dP7sum[ii]; }
        
        for (ii=0; ii<m; ii++){
            for (jj=0; jj<PP; jj++) {dctemp[jj] += (P7[ii]-Pavg)*(dP7[PP*ii+jj]-dPavg[jj]);}
        }

        for (ii=0; ii<PP; ii++){
            dcterm[ii] = (pow(P7sum,2)*dctemp[ii] - ctemp*P7sum*dP7sum[ii])/pow(P7sum,4);
        }
    }

	cterm = .5*ctemp/pow(P7sum,2);
	*Li = log(Pavg) + cterm;
    
    if(calcderiv==1){
        for (ii=0; ii<PP; ii++) {
            dLi[ii] = dP7sum[ii]/P7sum + dcterm[ii];
        }
    }

}
else {
	*Li = log(P7[0]);
    
    if(calcderiv==1){
    	for (ii=0; ii<PP; ii++) {
        	dLi[ii] = dP7[ii]/P7[0];
        }
    }
}

}


 /***********************************************************************************************
 *
 * CALCP7: 
 *
 *********************************************************************************************/

void calcp7(double *P7, int c1, int c7, double *P1, double *Bcons, double *Bex, double *coeff)

{

double *dP7=NULL;
double *dP1=NULL;
double *dcoeff=NULL;

calcpdp7(P7, dP7, c1, c7, P1, dP1, Bcons, Bex, coeff, dcoeff);

}

 /*********************************************************************************************
 *
 * CALCP1ALLND:
 *
 *********************************************************************************************/

void calcp1allnd(double *P1, double *U, double *uadd, double gam, int minput, int consindex)
{

double *dP1=NULL;
double *b=NULL;
double *excl=NULL;
double *X=NULL;
double *RD=NULL;
double *CM=NULL;
int *constants=NULL;
double dgam=0;

calcp1all(P1,dP1,U,b,excl,X,RD,CM,uadd,constants,gam,dgam,minput,consindex);

}


 /*********************************************************************************************
 *
 * CALCP7ALLND:
 *
 *********************************************************************************************/

void calcp7allnd(double *P7, double *U, double *Bcons, double *Bex, double *coeff, double *Y1, 
                    double *Y7, double *uadd, double gam, int minput, int consindex)
{

double *dP7=NULL;
double *b=NULL;
double *excl=NULL;
double *X=NULL;
double *RD=NULL;
double *CM=NULL;
double *dcoeff=NULL;
int *constants=NULL;
double dgam=0;

calcp7all(P7,dP7,U,b,excl,X,RD,CM,Bcons,Bex,coeff,dcoeff,Y1,Y7,uadd,constants,gam,dgam,minput,consindex);

}



