/***************************************************************************
 *
 * OEDTOOLS.C: Implementation of functions in oedtools.h.
 *
 * Matthew Gentzkow
 * July 11, 2003
 * Online Editions
 *
 ****************************************************************************/

#include <math.h>
#include "mex.h"
#include "oedcalc.h"
#include "oedcalcprob.h"
#include "oedtools.h"
#include "randomlib.h"


/**********************************************************************************
 *
 * PROBABILITIES7: 
 *
 ***********************************************************************************/
 
void probabilities7(double *P, double *X, double *Y1, double *Y7, double *RD, double *b, double *excl, 
				 double *CM, double *Bcons, double *Bex, int minput, int ninput)

{
double *P7, *U, gam, uadd[CC-1], coeff[5];
int ii, jj;

U = mxCalloc(minput*ninput*(CC-1),sizeof(double));
P7 = mxCalloc(minput,sizeof(double));

calcnonlin(coeff,NULL,&gam,NULL,uadd,b,CM);
calcu(U,b,excl,X,RD,CM,minput,ninput);

/* main evaluation routine */
for(ii=0; ii<ninput; ii++) {
	for(jj=0;jj<minput;jj++){P7[jj]=0;}
    calcp7allnd(P7,U,Bcons,Bex,coeff,Y1,Y7,uadd,gam,minput,ii);
	for(jj=0;jj<minput;jj++){P[ninput*jj+ii] = P7[jj];}
}
}


/**********************************************************************************
 *
 * PROBABILITIES1:
 *
 ***********************************************************************************/
 
void probabilities1(double *P1, double *X, double *RD, double *b, double *excl, double *CM, int minput, int ninput)
{

double *U, gam, uadd[CC-1];
int ii;

U = mxCalloc(minput*ninput*(CC-1),sizeof(double));
calcu(U,b,excl,X,RD,CM,minput,ninput);
calcnonlinsm(&gam,uadd,b,CM);
for (ii=0; ii<ninput*minput*CC; ii++) {P1[ii]=0;}

/* main evaluation routine */
for(ii=0; ii<ninput; ii++) {
    calcp1allnd(&P1[ii*minput*CC],U,uadd,gam,minput,ii);
}
}


/**********************************************************************
 *
 * CROSSDERIV: 
 *
 ***********************************************************************/

void crossderiv(double *CD, double *b, double *excl, int minput, int ninput, 
				double *X, double *RD, double *CM, double *mweights) 
{

double *P1, *P1news, P0, P0n, PA, PAn, PB, PBn, PAB, PABn, *U, weight;
double gam, uadd[CC-1];
int pindex;
int ii, jj, kk, rr, ss;

/* decide whether we're supposed to calculate the derivative */
int useweights = mweights==NULL ? 0:1;

P1 = mxCalloc(minput*CC,sizeof(double));
P1news = mxCalloc(minput*CC,sizeof(double));
U = mxCalloc(minput*ninput*(CC-1),sizeof(double));

calcnonlinsm(&gam,uadd,b,CM);
calcu(U,b,excl,X,RD,CM,minput,ninput);

/* initialize CD */
for(ii=0; ii<ninput*GG*GG; ii++) {
	CD[ii]=0;
}

/* main evaluation routine */
for(ii=0; ii<ninput; ii++) {

    /* calculate P1 (no-news day) as case gam==0 */
    calcp1allnd(P1,U,uadd,0,minput,ii);
    
    /* calculate P1news as case gam==1 */
    calcp1allnd(P1news,U,uadd,1,minput,ii);
    
	for(jj=0; jj<minput; jj++) {

        if(useweights==1) weight = mweights[ninput*jj+ii];
        pindex = CC*jj;
		
		/* loop over pairs of goods */
		for(kk=0; kk<GG; kk++) {
			for(rr=0; rr<GG; rr++) {
				P0=0; PA=0; PB=0; PAB=0;
                P0n=0; PAn=0; PBn=0; PABn=0;
				/* define relevant probabilities */
				for(ss=0; ss<CC; ss++) {
					if((CM[GG*ss+kk]==0) & (CM[GG*ss+rr]==0)){ 
                        P0 += P1[pindex+ss];
                        P0n += P1news[pindex+ss]; 
                    }
					if((CM[GG*ss+kk]==1) & (CM[GG*ss+rr]==0)) {
                        PA += P1[pindex+ss];
                        PAn += P1news[pindex+ss];
                    }
					if((CM[GG*ss+kk]==0) & (CM[GG*ss+rr]==1)) {
                        PB += P1[pindex+ss];
                        PBn += P1news[pindex+ss];
                    }
					if((CM[GG*ss+kk]==1) & (CM[GG*ss+rr]==1)) {
                        PAB += P1[pindex+ss];
                        PABn += P1news[pindex+ss];
                    }
				}
                
				if(useweights==1) {
                    CD[ii*GG*GG+kk*GG+rr] += 
                        weight*((1-gam)*(PAB*P0 - PA*PB)+gam*(PABn*P0n - PAn*PBn));
                } else {
            		CD[ii*GG*GG+kk*GG+rr] += (1-gam)*(PAB*P0 - PA*PB)+gam*(PABn*P0n - PAn*PBn);
                }
			}
		}
	}
	for (jj=0; jj<GG*GG; jj++) {CD[ii*GG*GG+jj] /= (double)minput;}
}

}


/***********************************************************************************************
 *
 * DELTAMAT: 
 *
 *********************************************************************************************/
 
void deltamat(double *DM, double *b, double *excl, int minput, double *X, double *RD, double *CM, 
				   double *Bcons, double *Bex, double *weights, double *Y1, double *Y7)
{

/*double G[NN*PP];*/
double *G;
int ii, jj, kk;

G = mxCalloc(NN*PP,sizeof(double));

gradients(G,b,excl,minput,X,RD,CM,Bcons,Bex,weights,Y1,Y7);
                   
/* initialize DM */
for(ii=0; ii<PP*PP; ii++){DM[ii]=0;}

/* calculate outer product */
for(ii=0; ii<NN; ii++) {
	for (jj=0; jj<PP; jj++) {
		for (kk=0; kk<PP; kk++) {
			DM[PP*jj+kk] += G[ii*PP+jj]*G[ii*PP+kk];
		}
	}
}

}



/**********************************************************************
 *
 * GRADIENTS:
 *
 ***********************************************************************/
 
void gradients( double *G, double *b, double *excl, int minput, double *X, double *RD, double *CM, 
				   double *Bcons, double *Bex, double *weights, double *Y1, double *Y7)
{

double coeff[5], dcoeff[5], uadd[CC-1];
double *P, *dP, *P1, *dP1, *U, *obsweights;
double Li, gam, dgam;
int c1, constants[GG];
int ii, jj, kk;

/* decide whether we're using 7-day or 1-day data */
int sevenday = Y7==NULL ? 0:1;

P = mxCalloc(minput,sizeof(double));
dP = mxCalloc(minput*PP,sizeof(double));
U = mxCalloc(minput*NN*(CC-1),sizeof(double));
obsweights = mxCalloc(minput,sizeof(double));

if(sevenday==0){
    P1 = mxCalloc(minput*CC,sizeof(double));
    dP1 = mxCalloc(minput*CC*PP,sizeof(double));
}

calcu(U,b,excl,X,RD,CM,minput,NN);
calcnonlin(coeff,dcoeff,&gam,&dgam,uadd,b,CM);
findconstants(constants,excl,X);

/* initialize G */
for(ii=0; ii<NN*PP; ii++){G[ii]=0;}

/* main evaluation routine */
for(ii=0; ii<NN; ii++) {
	for(jj=0;jj<minput;jj++){
        P[jj]=0;
        obsweights[jj] = weights[NN*jj+ii];
    }
    if(sevenday==1) {
        calcp7all(P,dP,U,b,excl,X,RD,CM,Bcons,Bex,coeff,dcoeff,Y1,Y7,uadd,constants,gam,dgam,minput,ii);
    } else {
        calcp1all(P1,dP1,U,b,excl,X,RD,CM,uadd,constants,gam,dgam,minput,ii);
        /* copy observed choice from P1 and dP1 into P and dP */
        c1=(int)Y1[ii];
        for(jj=0;jj<minput;jj++){
            P[jj]=P1[jj*CC+c1];
            for(kk=0;kk<PP;kk++){
                dP[jj*PP+kk]=dP1[jj*CC*PP+c1*PP+kk];
            }
        }
    }
	calcL(&Li,&G[ii*PP],P,dP,obsweights,minput);
}

}



/*********************************************************************************************
 *
 * ISDRAWS: 
 *
 ************************************************************************************************/

void isdraws( double *P0, double *RD, double *b, double *excl, int m, double *Y1, 
				double *Y7, double *X, double *CM, double *Bcons, double *Bex, int IJseed, int KLseed) 
{

double P1[CC], coeff[5], data[KK+DD], U[CC-1], uadd[CC-1];
double P7, udraw, gam, *c1, *c7;
char errstring[400];
int accept, counter;
int ii, jj, tt;	

calcnonlin(coeff,NULL,&gam,NULL,uadd,b,CM);

/* initialize RD generator */
RandomInitialize(IJseed,KLseed);

/* main evaluation routine */
for(ii=0; ii<NN; ii++) {

    for(jj=0; jj<m; jj++) {
		accept=0; counter=0; /* counter is just for testing */
		c1 = &Y1[ii];
		c7 = &Y7[ii];
		for(tt=0;tt<KK;tt++){data[tt]=X[KK*ii+tt];}
        
		while(accept==0){
			for(tt=0;tt<DD;tt++){
				data[KK+tt] = RandomGaussian(0,1);
			}

			calcusingle(U, b, excl, data, CM);
			calcp7allnd(&P7,U,Bcons,Bex,coeff,c1,c7,uadd,gam,1,0);

			udraw = RandomUniform();
			if(udraw<=P7) {
				accept=1;
				for(tt=0;tt<DD;tt++){RD[NN*DD*jj+DD*ii+tt]=data[KK+tt];}
				P0[NN*jj+ii]=P7;
			} else {
				counter++;
			}
			if(counter==100000 & jj==0){
                sprintf(errstring,"Importance sampling failed to find draw--probability of observed choice close to 0 (i=%d).",ii);
                counter=0;
                mexErrMsgTxt(errstring);
			}
		}	
	}
}
}







