/**********************************************************************
 *
 * TESTRAND: Test random number generator. If the generator is working
 *			 properly, the output should be:
 *
 *				6533892.0  
 *				14220222.0  
 *				7275067.0
 *				6172232.0  
 *				8354498.0   
 *				10633180.0
 *
 * Matthew Gentzkow
 * July 16, 2003
 * Online editions
 * 
 * MATLAB prototype:
 * draws = testrand()
 *
 * compiler command: 
 *	mex filename.c othfile1.c othfile2.c
 *
 * note: a*b*c in dimension means vectors stacked as a groups of b groups of c
 *
 * INPUTS:
 *
 * OUTPUTS:
 *	draws		6				test draws
 *
 ***********************************************************************/


#include <math.h>
#include "mex.h"
#include "randomlib.h"

#define NINPUTS 0
#define NOUTMAX 1

/* ****************************************************** */
/* mexFunction: MATLAB gateway routine                    */
/*														  */
/* remember: m is rows, n is columns					  */
/*													      */
/* ****************************************************** */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int nout[NOUTMAX], mout[NOUTMAX];
  int ii;
  double *OPTRS[NOUTMAX];
  double draw;


  /*set nout and mout arrays with output dimensions */
  mout[0] = 6;			nout[0] = 1;

  
  /* check number of arguments */
  if(nrhs != NINPUTS)
    mexErrMsgTxt("NINPUTS inputs required.");
  if(nlhs > NOUTMAX)
    mexErrMsgTxt("At most NOUTMAX outputs allowed.");

  /* initialize output elements */
  for (ii=0; ii< ((nlhs>0)? nlhs:1) ; ii++) {
	  plhs[ii] = mxCreateDoubleMatrix(mout[ii],nout[ii],mxREAL);
	  OPTRS[ii] = mxGetPr(plhs[ii]);
  }

  RandomInitialize(1802,9373);
  for (ii=0; ii<20000; ii++) {
	  draw = RandomUniform();
  }

  OPTRS[0][0] = 4096*4096*RandomUniform();
  OPTRS[0][1] = 4096*4096*RandomUniform();
  OPTRS[0][2] = 4096*4096*RandomUniform();
  OPTRS[0][3] = 4096*4096*RandomUniform();
  OPTRS[0][4] = 4096*4096*RandomUniform();
  OPTRS[0][5] = 4096*4096*RandomUniform();

}


