%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% BUILDMB.M: Function used in builddata to generate Bcons, Bex, Mcons, and Mincl matrices.
%
% Matthew Gentzkow
% July 17, 2003
% Online Editions
%
% note: a*b*c in dimension means vectors stacked as a groups of b groups of c
% note: for choices NOT included in CM (this can arise since CM will only include
%       choices actually observed in data) Bex(i,j)=j.
%
% INPUTS:
%	CC			scalar				number of choices
%	CM			CC x GG				i*j=1 if good j included in choice i
%	mindex		scalar				index of var with missing 1-day data (if any)
%	
% OUTPUTS:
%	Bcons		CC x CC				i,j=1 if choice j potentially consistent with choice i
%	Bex			CC x CC				i,j=k where 1-day choice i takes overall choice from j to k
%	Mcons		CC					first vector for missing 1-day data =k where k is other choice consistent w/ missing var
%	Mincl		CC					second vector for missing 1-day data =1 if choice i includes missing var
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Bcons, Bex, Mcons, Mincl] = buildMB(CC,CM,mindex)

Bcons = zeros(CC,CC);
Mcons = cumsum(ones(CC,1));
Bex = zeros(CC,CC);
Mincl = zeros(CC,1);
for i = 1:CC
    for j = 1:CC
        if CM(i,:)<=CM(j,:), Bcons(i,j)=1; end
        newchoice = findinmat(min(CM(i,:)+CM(j,:),1),CM);
		if newchoice>0
            Bex(i,j) = newchoice;
        else
            Bex(i,j)=j;
        end
    end
    
    if CM(i,mindex)==1
        Mincl(i)=1;
        tempv = CM(i,:);
        tempv(mindex)=0;
        ind = findinmat(tempv,CM);
        if ind>0, Mcons(ind)=i; end
    end
end

% switch to C indexing
Bex = Bex-1;
Mcons = Mcons-1;

% transpose matrices so dimensions go the right way
Bcons = Bcons';
Bex = Bex';
