%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% GENB0.M: Generate parameter starting values.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	X			NN x KK		 		observable rhs vars
%	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
%	T			PP x ZZ				constraint matrix
%	A			PP					constraint vector
%	CM			CC x GG				i,j=1 if good j included in choice i
%	Bcons		CC x CC				i,j=1 if choice j potentially consistent with choice i
%	Bex			CC x CC				i,j=k where 1-day choice i takes overall choice from j to k
%
%	
% OUTPUTS:
%	b0c			ZZ					(constrained) parameter vector
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function b0 = genb0(Y1,Y7,X,excl,T,A,CM,Bcons,Bex)

global ZZ m0 NN KK DD PP tolf0 tolff tolx0 tolxf mfe0 mfef...
       miter0 miterf msteps mf UseDyn

	OPTIONS = optimset('Display','iter','MaxFunEvals',mfe0,'MaxIter',miter0,'TolFun',tolf0,'TolX',tolx0,...
                        'GradObj','on','LargeScale','off');
b0c=zeros(ZZ,1);
RD = randn(NN*DD,m0);
weights0 = ones(NN,m0);

if UseDyn==1
	sprintf('Preliminary parameter estimation 1')
	tic
	b0c = fminunc('likelihood7',b0c,OPTIONS,1,excl,Y1,Y7,X',RD(:,1),CM',Bcons,Bex,T,A,weights0(:,1));	
	b0 = (b0c'*T'+A')'
	toc
    lastdone = 'Preliminary';
	save tempout b0 lastdone

    if m0>1
    	sprintf('Preliminary parameter estimation 2')
	    tic
	    b0c = fminunc('likelihood7',b0c,OPTIONS,m0,excl,Y1,Y7,X',RD,CM',Bcons,Bex,T,A,weights0);
	    b0 = (b0c'*T'+A')'
	    toc
        lastdone = 'Preliminary 2';
        save tempout b0 lastdone
    end
else
	sprintf('Preliminary parameter estimation 1')
	tic
	[b0c,L,exitflag,output,dLc,Hc] = fminunc('likelihood1',b0c,OPTIONS,1,excl,Y1,Y7,X',RD(:,1),CM',Bcons,Bex,T,A,weights0(:,1));	
	b0 = (b0c'*T'+A')'
	toc
    lastdone = 'Preliminary';
	save tempout b0 lastdone
end

