%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% GOODFIT.M: Calculate goodness of fit measure
%
% Matthew Gentzkow
% September 25, 2003
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	Y1		    NN			    		one-day choices
%   P1cond      NN x CC                 probability of each 1-day choice for each consumer with
%                                       expectation over unobservables conditional on 7-day choice
%	
% OUTPUTS:
%   probsyes    CC                      avg probability of choice for those who chose it
%   probsno     CC                      avg probability of choice for those who didn't choose it
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [probsyes,probsno] = goodfit(Y1,P1cond)

global CC NN

chose = zeros(CC,1);
nchose = zeros(CC,1);
notchose = zeros(CC,1);
for ii = 1:NN
	for jj = 1:CC
		if Y1(ii)+1==jj
			chose(jj) = chose(jj)+P1cond(ii,jj);
			nchose(jj) = nchose(jj)+1;
		else
			notchose(jj) = notchose(jj)+P1cond(ii,jj);
		end
	end
end
probsyes = chose./nchose;
probsno = notchose./(NN-nchose);
