%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% NUMCHOOSING.M: Calculate the number of consumers choosing each good / bundle
%
% Matthew Gentzkow
% September 25, 2003
% Online Editions
%
% note: a*b*CC in dimension means vectors stacked as a groups of b groups of CC
%
% INPUTS:
%	Y1			NN					one-day observed choices
%	CM			CC x GG 		    i x j=1 if choice i includes good j
%	weights		NN					weights on each observation
%	nn			scalar				let this be an input so it can vary
%	
% OUTPUTS:
%	nchgood 	GG					number of consumers choosing each good
%	nchbundle	CC					number of consumers choosing each bundle
%   Y1dc        nn x CC             i x j=1 if consumer i chose bundle j and 0 otherwise
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [nchgood,nchbundle,Y1dc] = numchoosing(Y1,CM,weights,nn)

global GG CC KK DD PP ZZ msee

nchbundle = zeros(CC,1);
nchgood = zeros(GG,1);

Y1 = Y1+1;						    % so can use Y1 as matrix indices
Y1dc = zeros(nn,CC);				% Y1dc is a NN x CC matrix of dummy vars for choices
for ii = 1:CC
	Y1dc(:,ii) = 1-min(abs(Y1-ii),1);
	nchbundle(ii) = sum(Y1dc(:,ii).*weights);
end

for ii = 1:CC
	for jj = 1:GG
		if(CM(ii,jj)==1), nchgood(jj) = nchgood(jj)+nchbundle(ii); end
	end
end
