
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% TESTIS.M: Compare variance of importance sampling and standard estimates of probabilities.
%
% Matthew Gentzkow
% July 16, 2003
% Online Editions
%
% note: a*b*c in dimension means vectors stacked as a groups of b groups of c
%
% INPUTS:
%	X			NN x KK				observable rhs vars
%	Y1			NN					one-day choices
%	Y7			NN					seven-day choices
%	CM			CC*GG				i*j=1 if good j included in choice i
%	Bcons		CC*CC				i*j=1 if choice i potentially consistent with choice j
%	Mcons		CC					first vector for missing 1-day data
%	Mincl		CC					second vector for missing 1-day data
%	Bex			CC*CC				i*j=k where 1-day choice j takes overall choice from i to k
%	b			PP					parameter vector
%	excl		PMAX				vector of exclusion restrictions (=1 if excluded)
%	mistest		scalar				number of monte carlo simulations
%   meach       scalar              number of simulation draws to use in each prob estimate
%   b0          PP                  preliminary parameter estimate to calculate IS draws
%
%	
% OUTPUTS:
%	varis		scalar	    		average variance of imp sampling simulated probabilities
%	varstd		scalar				average variance of standard simulated probabilities
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [varis,pavgis,varstd,pavgstd] = testis(X,Y1,Y7,CM,Bcons,Mcons,Mincl,Bex,b,excl,mistest,meach,b0)

global NN DD mis;

% calculate preliminary probabilities Pavg0
RD = randn(NN*DD,mis);
Pavg0 = avgprob(b0,excl,mis,Y1,Y7,X',RD,CM',Bcons,Mcons,Mincl,Bex);

Pstd = zeros(NN,mistest);
Pis = zeros(NN,mistest);

for ii = 1:mistest
    
    RDstd = randn(NN*meach*DD,1);
    [RDis, weights, P0] = genis(X,Y1,Y7,CM,Bcons,Mcons,Mincl,Bex,b0,excl,meach,Pavg0);
    weights = reshape(weights,NN,meach);
    
    Pcur = probabilities7(X,Y1,Y7,RDstd,b,excl,CM,Bcons,Mcons,Mincl,Bex,meach,NN);
    Pcur = reshape(Pcur,NN,meach);
    Pstd(:,ii) = mean(Pcur,2);
    
    Pcur = probabilities7(X,Y1,Y7,RDis,b,excl,CM,Bcons,Mcons,Mincl,Bex,meach,NN);
    Pcur = reshape(Pcur,NN,meach).*weights;
    Pis(:,ii) = mean(Pcur,2);

end

varis = mean(var(Pis'));
pavgis = mean(Pis,2);
varstd = mean(var(Pstd'));
pavgstd = mean(Pstd,2);
