%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% TRANSMATS.M: Generate transformation matrix and vector (T and A) from linear
%				constraint Rb=r.
%
% Matthew Gentzkow
% September 16, 2002
% Online Editions
%
% note: a*b*c in dimension means vectors stacked as a groups of b groups of c
%
% INPUTS:
%	R			(PP-ZZ) x PP		matrix of linear restrictions
%	r			(PP-ZZ)				vector of linear restrictions (Rb=r)
%	
% OUTPUTS:
%	T			PP x ZZ				constraint matrix
%	A			PP					constraint vector
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [T,A] = transmats(R,r)

global PP ZZ;

if ~isempty(R)
    Pmat = eye(PP)-R'*inv(R*R')*R;
    T = orth(Pmat);
    L = null(Pmat);
    A = r'*inv(L'*R')*L';
else
    T = eye(PP);
    A = zeros(1,PP);
end

% test transmats
tolerance = 10^(-12);
b0 = randn(1,PP);
bc = b0*T;
b1 = bc*T'+A;
bc = b1*T;
b2 = bc*T'+A;
if abs(max(b2-b1))>tolerance | abs(min(b2-b1))>tolerance
    error('Error in transmats');
end

A = A';