import pandas as pd
import csv
import sys
sys.path.append('source/lib/')
from load_yaml import load_yaml

# globals
paths = load_yaml('config_global.yaml')


def main():
    rep_path = paths['build']['analysis']['ams_2011']['replication']
    exp_path = paths['build']['analysis']['ams_2011']['experiment']

    # Read in file
    fname = '%s/prog_n.dat' % rep_path
    data = [line.split() for line in open(fname, 'rb')]
    data = join_every_n_list(data, 6)
    columns = [
        "famid",
        "state",
        "school",
        "age",
        "yrs",
        "gender",
        "town",
        "nchs98",
        "nch_sc",
        "orderofbirth",
        "a",
        "b",
        "c",
        "d",
        "fate1",
        "fate2",
        "fate3",
        "fate4",
        "fate5",
        "mote1",
        "mote2",
        "mote3",
        "mote4",
        "mote5",
        "indig",
        "faho",
        "moho",
        "denti1",
        "denti2",
        "denti3",
        "denti4",
        "denti5",
        "denti6",
        "poor1",
        "poorc",
        "inc"
    ]
    data = pd.DataFrame(data, columns=columns)

    # Write to csv
    outfile = '%s/prog_n.csv' % exp_path
    data.to_csv(outfile, index=False)
    return None


def join_every_n_list(l, n):
    '''
    Given a list of lists l, join every n lists together in one list.
    Raise ValueError if len(l) is not divisble by n.
    '''
    if len(l) % n != 0:
        raise ValueError
    out_l = []
    for i in range(0, len(l), n):
        temp_l = []
        for j in l[i:i + n]:
            temp_l += j
        out_l.append(temp_l)
    return out_l


main()
