import subprocess
import os
import shutil
# Remember that this has to be loaded on Sherlock. http://sherlock.stanford.edu/mediawiki/index.php/Software (search: ml load python/2.7.5)
import pandas as pd
import sys


def main():
    '''
    Write out scores and hessian matrix to files. 
    '''
    set_program_environment()
    score_df = clean_score('score.dat')
    score_df = merge_score(score_df, 'obs_identifier.out')
    hessian_df = clean_hessian('gslab_hess.dat', 'obs_identifier.out')

    # write to files
    score_df.to_csv('score_df.csv', sep='\t', index=False)
    hessian_df.to_csv('hess_df.csv', sep='\t', header=False, index=False)

    return None


def set_program_environment():
    '''
    Compile and run simulation program. 
    Change to output directory. 
    '''
    subprocess.check_call(['sh', 'compile_and_run.sh'])
    os.chdir('../../../../build/analysis/ams_2011/score_hessian')
    return None


def clean_score(score_path):
    '''
    Read raw score matrix and clean up the format
    '''
    score_list = pd.read_csv(score_path, delim_whitespace=True, header=None).values.flatten()
    Num_par = score_list[-3]
    score_list = score_list[:len(score_list) - 3]
    score_matrix = score_list.reshape(int(len(score_list) / Num_par), int(Num_par))
    score_df = pd.DataFrame(score_matrix)
    return score_df


def merge_score(score_df, identifier_path):
    '''
    Merge score dataframe with person identifier
    '''
    identifiers = pd.read_csv(identifier_path, delimiter=',', header=None)
    names = ['family_id', 'birth_order'] + ['score ' +
                                            str(x) for x in range(1, score_df.shape[1] + 1)]
    score_df = pd.concat([identifiers, score_df], axis=1)
    score_df.columns = names
    return score_df


def clean_hessian(hess_path, identifier_path):
    '''
    Read raw Hessian matrix, clean up the format, divide by 
    number of observations for Hessian of average log likelihood.
    '''
    hess_list = pd.read_csv(hess_path, delim_whitespace=True, header=None).values.flatten()
    num_par = int((len(hess_list)) ** (.5))
    hess_matrix = hess_list.reshape(num_par, num_par)
    n_obs = len(pd.read_csv(identifier_path, delimiter=',', header=None))
    hess_matrix = hess_matrix / (n_obs ^ 2)  # get Hessian of average likelihood
    hess_df = pd.DataFrame(hess_matrix)
    return hess_df


# Execute
main()
