function compute_counterfactual
    addpath(genpath('build/analysis/gentzkow_2007/replication/n_full'))
    addpath(genpath('source/analysis/gentzkow_2007'))
    global NN NNtot GG CC KK DD PP PMAX ZZ msee UseDyn
    
    % Load input variables   
    load('build/external/gentzkow_2007/input/run121905', ...
        'R', 'r', 'buildparams')
    
    load('build/external/gentzkow_2007/results/OEDest121905', ...
         'bstar', 'estparams');

    load('build/external/gentzkow_2007/results/OEDout121905', 'RD');

    if (wrongbuildparams(buildparams))
        error('Parameters in data do not match those with which C-files were compiled');
    end
 
    % Set the values of global variables
    [NNtot, NN, GG, CC, KK, DD, PMAX, PP, ZZ] = extract(buildparams);
    UseDyn = extract(estparams);
    draw_dimensions = size(RD);
    msee = draw_dimensions(2); % Number of random draws

    % Transform parameters
    ii = PP-1:PP;   
    bstar(ii) = log(bstar(ii)) - log(1 - bstar(ii));
    [T, A] = transmats(R, r);
    bstarc = (bstar'*T)';

    % Compute the counterfactual at the (transformed) estimated parameters
    cfac_func = @(Beta) cfac(Beta, T, A, RD);
    [cf, Cdep] = cfac_func(bstarc);
   
    % Check that the computed counterfactuals are equal to the published ones
    load('build/external/gentzkow_2007/results/OEDout121905', ...
         'D', 'Dse');
    load('build/external/gentzkow_2007/results/OEDwel121905', ...
         'drevyear', 'drevyearse');
    assertElementsAlmostEqual([D(2,1); drevyear(2, 1)], cf, 'relative', 10e-8)

    % Save the counterfactual along with its SE and Jacobian
    cf_se = [Dse(2,1); drevyearse(2,1)];
    cf_jacobian = NumJacob(@(bc) cfac_func(bc), bstarc, 10E-6);
    save('build/analysis/gentzkow_2007/counterfactual', ...
         'cf', 'cf_se', 'cf_jacobian', 'Cdep');

    exit
end

function [cf, Cdep] = cfac(bstarc, T, A, RD)
    global NN NNtot GG CC KK DD PP PMAX ZZ msee UseDyn

    % Load data   
    load('build/external/gentzkow_2007/input/run121905', ...
         'Y1all', 'Y7all', 'Xall', 'CM', 'surveyweights', 'ascribed',...
         'excl', 'Bcons', 'Bex')

    bstar = (bstarc'*T' + A')';
    % Transform parameters back
    ii = PP-1:PP;   
    bstar(ii) = exp(bstar(ii)) ./ (1 + exp(bstar(ii)));
    
    % Take out ascribed observations
    Y1 = selectinmat(Y1all,1 - ascribed);
    Y7 = selectinmat(Y7all,1 - ascribed);
    X  = selectinmat(Xall, 1 - ascribed);
    
    [~, ~, ~, ~, mweights, ~, ~] = ...
        probweight(Xall, Y1all, Y7all, X, Y1, Y7, RD, bstar, excl, CM, Bcons, ...
                   Bex, msee, UseDyn, ascribed, surveyweights);
    
    % Calculate number choosing each good/bundle
    [~, nchbundle, Y1dc] = numchoosing(Y1all, CM, surveyweights, NNtot);
    
    % Calculate distribution of second choices & changes in demand
    % demchanges_chat_dep.m is a script copied from the replication files
    % which has been modified to output the counterfactual at each
    % observation Cdep
    [~, ~, D, ~, Cdep] = demchanges_chat_dep(nchbundle, Y1dc, bstar, bstar, excl, ...
                                             Xall, RD, CM, surveyweights, mweights, 1);

                                
    piweb   = .1679;   % Profit per web reader
    piprint = .780919; % Profit per print reader
    goodspi = [piprint piweb piprint];  
    issuesperyear = [365-104 365 365-104];
    
    drevday  = D .* repmat(goodspi, GG, 1);
    drevyear = drevday .* repmat(issuesperyear, GG, 1);

    cf = [D(2,1); drevyear(2,1)];
    Cdep = Cdep(:,1,2);
    
end

