version 13
set more off

program main
    insheet using "build/external/gentzkow_2007/raw data/ScarDC.txt", names

    replace wued = 0 if wued == .
    replace wuwork = 0 if wuwork == .
    tab missing, gen(zmis)

    gen npostboth7  = ndpost7*nwpost7
    gen ndpostonly7 = ndpost7 & ~nwpost7
    gen nwpostonly7 = ~ndpost7 & nwpost7

	gen omitted = ascribed | nnonews
	
	gen kept_cons = (1-omitted)
	gen omit_cons = 1*omitted
	
	* Interact omitted with each panel variable
	foreach var of varlist ndpost1 ndpost7 nwpost1 nwpost7 npostboth7 ndpostonly7 nwpostonly7{
		gen kept_`var' = `var'*(1-omitted)
		gen omit_`var' = `var'*(omitted)	
	}
	
	* Interact omitted with each of the controls	
	foreach var of varlist ndtimes7 page_m psex phs pcol pgrad pjcomp pjwhitecol ///
        pempft zmis2-zmis8 hloginc_m vdem vrep date_* hzone_* wnouse30{
		gen kept_`var' = `var'*(1-omitted)
			
	}
	
	foreach var of varlist ndtimes7 page_m psex phs pcol pgrad pjcomp pjwhitecol ///
        pempft zmis2-zmis8 hloginc_m vdem vrep date_* hzone_* wnouse30{
		gen omit_`var' = `var'*(omitted)	
	}
	
	* Interact omitted with each of the instruments
	foreach var of varlist wlwork wued wuwork wcfast{
		gen kept_`var' = `var'*(1-omitted)
		gen omit_`var' = `var'*(omitted)	
	}
	
	local kept_controls  kept_ndtimes7 kept_page_m     kept_psex   kept_phs     kept_pcol ///
		  kept_pgrad     kept_pjcomp   kept_pjwhitecol kept_pempft kept_zmis2-kept_zmis8 ///
		  kept_hloginc_m kept_vdem     kept_vrep       kept_date_* kept_hzone_* kept_wnouse30
		
	local omit_controls  omit_ndtimes7 omit_page_m     omit_psex   omit_phs     omit_pcol ///
		  omit_pgrad     omit_pjcomp   omit_pjwhitecol omit_pempft omit_zmis2-omit_zmis8 ///
		  omit_hloginc_m omit_vdem     omit_vrep       omit_date_* omit_hzone_* omit_wnouse30
	
    ivregress 2sls ndpost1 `kept_controls' kept_cons ///
			  omit_nwpost1 `omit_controls' omit_cons ///
			 (kept_nwpost1 = kept_wlwork kept_wued kept_wuwork kept_wcfast), robust nocon
    gen_output, spec_name(iv_1day)
    
    ivregress 2sls ndpost7 `kept_controls' kept_cons ///
			  omit_nwpost7 `omit_controls' omit_cons ///
			 (kept_nwpost7 = kept_wlwork kept_wued kept_wuwork kept_wcfast), robust nocon
    gen_output, spec_name(iv_5day)

    reg ndpost1 kept_nwpost1 `kept_controls' ///
				omit_nwpost1 `omit_controls', robust nocon
    gen_output, spec_name(ols_1day)
    
    reg ndpost7 kept_nwpost7 `kept_controls' kept_cons ///
				omit_nwpost1 `omit_controls' omit_cons , robust nocon
    gen_output, spec_name(ols_5day)

    reg ndpost1 kept_nwpost1 kept_npostboth7 kept_ndpostonly7  kept_nwpostonly7 `kept_controls' kept_cons ///
			    omit_nwpost1 omit_npostboth7 omit_ndpostonly7  omit_nwpostonly7 `omit_controls' omit_cons , robust nocon
    gen_output, spec_name(ols_panel)
		
	outsheet using "build/analysis/gentzkow_2007/data.csv", comma replace
end

program gen_output
    syntax, spec_name(str)
    matrix beta = e(b)
    matrix vcov = e(V)
    local out_path "build/analysis/gentzkow_2007/beta_vcov"
    matrix_to_txt, saving(`out_path'/`spec_name'_beta.txt) ///
        mat(beta) format(%19.0g) replace
    matrix_to_txt, saving(`out_path'/`spec_name'_vcov.txt) ///
        mat(vcov) format(%19.0g) replace
end

main
