function compute_likelihood
    analysis = 'build/analysis/gentzkow_2007';
    data = 'build/external/gentzkow_2007';
    
    addpath(genpath(sprintf('%s/replication/n_1', analysis)))
    
    load(sprintf('%s/input/run121905.mat', data))
    load(sprintf('%s/results/OEDestRD121905', data), 'RD', 'weights')           
    load(sprintf('%s/results/OEDest121905', data), 'bstar', 'L', 'S3')       
    
    global UseDyn ImpSamp NN NNtot GG CC KK DD PP PMAX ZZ CCW ...
           tolf0 tolff tolx0 tolxf mfe0 mfef miter0 miterf ...
           m0 mis msteps mf mtest msee melast
    [NNtot, NN, GG, CC, KK, DD, PMAX, PP, ZZ, CCW, XX, UseDyn] = ...
        extract(buildparams);
    [ImpSamp, tolf0, tolff, tolx0, tolxf, mfe0, mfef, miter0, miterf, ...
     m0, mis, msteps, mf, mtest, msee, melast] = setestparams;
    
    % Transform and rename relevant variables
    ii = PP-1:PP;   
    bstar_tr = bstar;   % transformed parameters (published)
    bstar_est = bstar;  % untransformed parameters (used in estimation)
    bstar_est(ii) = log(bstar_est(ii)) - log(1 - bstar_est(ii));
    vcov = S3;
    
    % Prepare data
    Y1 = selectinmat(Y1all, 1 - ascribed);
    Y7 = selectinmat(Y7all, 1 - ascribed);
    X = selectinmat(Xall,   1 - ascribed);
    [T, A] = transmats(R, r);
    
    % Calculate log likelihood and score vector
    score = zeros(NN, ZZ);
    Li = zeros(NN, 1);
    bstarc = (bstar_est'*T)';
    for i = 1:NN
        [Li(i), dLci] = likelihood7(bstarc, mf, excl, Y1(i, :), Y7(i, :), ...
                                    X(i, :)', RD(DD*(i - 1) + 1:DD*i, :), ...
                                    CM', Bcons, Bex, T, A, weights(i, :));
        score(i,:) = -dLci';
    end
    loglik = -Li;
    
    assert(abs(L - sum(loglik)) < 10E-4)

    % Delete the checksum log if it exists
    checksum_log = sprintf('%s/likelihood_checksum.log', analysis);
    if exist(checksum_log, 'file') == 2
        delete(checksum_log);
    end
    % Write the log likelihood to the checksum log
    diary(checksum_log)
    disp(['Log Likelihood: ' num2str(sum(loglik))])
    diary off
    
    % Correct variance-covariance matrix (outputs untransformed)
    df_orig = 1 ./(1 + exp(bstar_tr(ii))) .^ 2;
    vcov(ii, :) = vcov(ii, :) ./ repmat(df_orig, 1, PP);
    vcov(:, ii) = vcov(:, ii) ./ repmat(df_orig', PP, 1);
    
    save(sprintf('%s/gentzkow_aer_2007', analysis), 'score', 'vcov', 'T');
    exit
end
