%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% SELECTINMAT.M: Select a subset of the rows of a matrix and return a new matrix.
%			with only the selected rows. Inputs are the matrix from which to 
%			select and a vector =1 if the observation is to be retained or a 
%			zero otherwise.
%
% Matthew Gentzkow
% October 8, 2002
% Online Editions
%
% note: a*b*c in dimension means vectors stacked as a groups of b groups of c
%
% INPUTS:
%	M			matrix
%	V			V(i)=1 if row i to be retained, =0 otherwise
%	
% OUTPUTS:
%	Mnew		matrix of only the selected rows of M
%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Mnew = selectinmat(M,V)

[n,k] = size(M);
if n ~= size(V), error('Matrix and vector of different sizes in SELECTINMAT'); end

indices = find(V);
Mnew = M(indices,:);
