import shutil
import os
import re
import gslab_scons as gs

def main():
    '''Install the Gentzkow (2007) replication library'''
    analysis   = 'analysis/gentzkow_2007'
    source_dir = 'source/%s/gentzkow_aer_2007_orig/program' % analysis
    repl_files = get_replication_files(source_dir)
    dest_dir   = 'build/%s/replication' % analysis

    # Copy the code directory to build
    nfull_path = os.path.join(dest_dir, 'n_full')
    if os.path.isdir(nfull_path):
        shutil.rmtree(nfull_path)
    shutil.copytree(source_dir, nfull_path)

    # Make NN = 1 and NN = 15372 versions of the directory
    n1_path = re.sub('n_full', 'n_1', nfull_path)
    if os.path.isdir(n1_path):
        shutil.rmtree(n1_path)    
    shutil.copytree(nfull_path, n1_path)

    n1_param_path = os.path.join(n1_path, 'c files/include/oedcalc.h')
    with open(n1_param_path, 'rU') as f:
        lines = f.readlines()

    line = [l for l in lines if re.search('^#define NN', l)][0]
    line_no = lines.index(line)
    lines[line_no] = re.sub('#define NN [0-9]+', '#define NN 1', line)

    with open(n1_param_path, 'wb') as f:
        f.writelines(lines)

    # Fix source directories in mexall.m files
    n1_mex_path = os.path.join(n1_path, 'mexall.m')
    edit_mexall(n1_mex_path)
    nfull_mex_path = os.path.join(nfull_path, 'mexall.m')
    edit_mexall(nfull_mex_path)

    # Compile each directory
    nfull_files = ['%s/%s' % (nfull_path, f) for f in repl_files]
    n1_files    = ['%s/%s' % (n1_path, f)    for f in repl_files]
    # Make mex directories
    for p in ['%s/c files/mex' % base for base in [nfull_path, n1_path]]:
        os.makedirs(p)

    gs.build_matlab(source = '%s/mexall.m' % nfull_path,
                    target = nfull_files, env = {})

    gs.build_matlab(source = '%s/mexall.m' % n1_path,
                    target = n1_files, env = {})


def get_replication_files(path):
    wanted = []
    exts = ['m', 'h', 'c'] # File extensions of files of interest
    good_ext = lambda x: bool([True for e in exts if re.search('\.%s$' % e, x)])
    for item in os.walk(path):
        (root, dirs, files) = item
        files = ['%s/%s' % (root, f) for f in files if good_ext(f)]
        wanted += files

    wanted = [re.sub('%s\/?' % path, '', f) for f in wanted]
    return wanted


def edit_mexall(path):
    # Fix source directories in mexall.m files
    with open(path, 'rU') as f:
        lines = f.readlines()

    line = [l for l in lines if re.search('^sourcepath =', l)][0]
    line_no = lines.index(line)

    sourcepath = re.sub('mexall\.m', 'c files/source', path)
    lines[line_no] = "sourcepath = '%s';\n" % sourcepath

    with open(path, 'wb') as f:
        f.writelines(lines)


if __name__ == '__main__':
    main()
